@@ -38,7 +38,10 @@ equality.
 
 =cut
 
-use Moo;
+use Moo 1.000003;
+
+# screw you PAUSE
+our $VERSION = '1.59';
 
 with qw(
   SQL::Translator::Role::Error
